﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using Newtonsoft.Json;
using System.IO;

namespace Algomojo
{
    public class API
    {
        private const String UserAgent = "Algomojo Wrapper 1.0";


        public class Request
        {
        }

        private class InternalRequest
        {
            public string api_key { get; set; }
            public string api_secret { get; set; }
            public Request data { get; set; }
        }

        public class PlaceOrderRequest : Request
        {
            public string strg_name { get; set; }   //   Strategy Name            
            public string s_prdt_ali { get; set; }   //   Product alias(BO:BO||CNC:CNC||CO:CO||MIS:MIS||NRML:NRML)
            public string Tsym { get; set; }   //   Trading Symbol
            public string exch { get; set; }   //   Exchange name
            public string Ttranstype { get; set; }   //   Transtaction Type
            public string Ret { get; set; }   //   Retention
            public string prctyp { get; set; }   //   Price Type
            public string qty { get; set; }   //   Quantity
            public string discqty { get; set; }   //   Disclosed Quantity
            public string MktPro { get; set; }   //   Market Pro
            public string Price { get; set; }   //   Price
            public string TrigPrice { get; set; }   //   Trigger Price
            public string PCode { get; set; }   //   Product Code
            public string AMO { get; set; }   //   AMO

            public PlaceOrderRequest(string strg_name, string s_prdt_ali, string Tsym, string exch, string Ttranstype, string Ret, string prctyp, string qty, string discqty, string MktPro, string Price, string TrigPrice, string PCode, string AMO)
            {                
                this.strg_name = strg_name;                
                this.s_prdt_ali = s_prdt_ali;
                this.Tsym = Tsym;
                this.exch = exch;
                this.Ttranstype = Ttranstype;
                this.Ret = Ret;
                this.prctyp = prctyp;
                this.qty = qty;
                this.discqty = discqty;
                this.MktPro = MktPro;
                this.Price = Price;
                this.TrigPrice = TrigPrice;
                this.PCode = PCode;
                this.AMO = AMO;
            }
        }

        public class OrderData
        {
            public string strg_name { get; set; }   //   Strategy Name
            public string order_refno { get; set; }   //   Order Reference Number
            public string user_apikey { get; set; }   //   User API Key
            public string api_secret { get; set; }   //   API Key Secret
            public string s_prdt_ali { get; set; }   //   Product alias(BO:BO||CNC:CNC||CO:CO||MIS:MIS||NRML:NRML)
            public string Tsym { get; set; }   //   Trading Symbol
            public string exch { get; set; }   //   Exchange name
            public string Ttranstype { get; set; }   //   Transtaction Type
            public string Ret { get; set; }   //   Retention
            public string prctyp { get; set; }   //   Price Type
            public string qty { get; set; }   //   Quantity
            public string discqty { get; set; }   //   Disclosed Quantity
            public string MktPro { get; set; }   //   Market Pro
            public string Price { get; set; }   //   Price
            public string TrigPrice { get; set; }   //   Trigger Price
            public string PCode { get; set; }   //   Product Code
            public string AMO { get; set; }   //   AMO

            public OrderData(string strg_name, string order_refno, string user_apikey, string api_secret, string s_prdt_ali, string Tsym, string exch, string Ttranstype, string Ret, string prctyp, string qty, string discqty, string MktPro, string Price, string TrigPrice, string PCode, string AMO)
            {
                this.strg_name = strg_name;
                this.order_refno = order_refno;
                this.user_apikey  = user_apikey;
                this.api_secret = api_secret;
                this.s_prdt_ali = s_prdt_ali;
                this.Tsym = Tsym;
                this.exch = exch;
                this.Ttranstype = Ttranstype;
                this.Ret = Ret;
                this.prctyp = prctyp;
                this.qty = qty;
                this.discqty = discqty;
                this.MktPro = MktPro;
                this.Price = Price;
                this.TrigPrice = TrigPrice;
                this.PCode  = PCode;
                this.AMO = AMO;
            }
        }

        public class PlaceMultiOrderRequest : Request
        {
            public OrderData[] orders { get; set; }

            public PlaceMultiOrderRequest(OrderData[] orders)
            {
                this.orders = orders;
            }
        }

        public class PlaceBOOrderRequest : Request
        {
            public string strg_name { get; set; }   //   Strategy Name            
            public string s_prdt_ali { get; set; }   //  Product alias(BO:BO||CNC:CNC||CO:CO||MIS:MIS||NRML:NRML)
            public string TokenNo { get; set; }   // Token Number
            public string exch { get; set; }   //    Exchange name
            public string Ttranstype { get; set; }   //  Transtaction Type
            public string Ret { get; set; }   // Retention
            public string qty { get; set; }   // Quantity
            public string discqty { get; set; }   // Disclosed Quantity
            public string Price { get; set; }   //   Price
            public string ltpOratp { get; set; }   // Trigger Price
            public string SqrOffAbsOrtick { get; set; }   //s SquareOff Absolute or Ticks
            public string SqrOffvalue { get; set; }   // Square Off Value
            public string SLAbsOrticks { get; set; }   //    Stoploss Absolute or Ticks
            public string SLvalue { get; set; }   // Stoploss Value
            public string trailingSL { get; set; }   //  Trailing Stoploss
            public string tSLticks { get; set; }   //    Trailing Stoploss Ticks

            public PlaceBOOrderRequest(string strg_name, string s_prdt_ali, string TokenNo, string exch, string Ttranstype, string Ret, string qty, string discqty, string Price, string ltpOratp, string SqrOffAbsOrtick, string SqrOffvalue, string SLAbsOrticks, string SLvalue, string trailingSL, string tSLticks)
            {                
                this.strg_name = strg_name;                
                this.s_prdt_ali = s_prdt_ali;
                this.TokenNo = TokenNo;
                this.exch = exch;
                this.Ttranstype = Ttranstype;
                this.Ret = Ret;
                this.qty = qty;
                this.discqty = discqty;
                this.Price = Price;
                this.ltpOratp = ltpOratp;
                this.SqrOffAbsOrtick = SqrOffAbsOrtick;
                this.SqrOffvalue = SqrOffvalue;
                this.SLAbsOrticks = SLAbsOrticks;
                this.SLvalue = SLvalue;
                this.trailingSL = trailingSL;
                this.tSLticks = tSLticks;
            }
        }

        public class BOOrderData
        {
            public string order_refno { get; set; }   // Order Reference Number
            public string strg_name { get; set; }   //   Strategy Name
            public string user_apikey { get; set; }   // User API Key
            public string api_secret { get; set; }   // API Secret Key
            public string s_prdt_ali { get; set; }   //  Product alias(BO:BO||CNC:CNC||CO:CO||MIS:MIS||NRML:NRML)
            public string TokenNo { get; set; }   // Token Number
            public string exch { get; set; }   //    Exchange name
            public string Ttranstype { get; set; }   //  Transtaction Type
            public string Ret { get; set; }   // Retention
            public string qty { get; set; }   // Quantity
            public string discqty { get; set; }   // Disclosed Quantity
            public string Price { get; set; }   //   Price
            public string ltpOratp { get; set; }   // Trigger Price
            public string SqrOffAbsOrtick { get; set; }   //s SquareOff Absolute or Ticks
            public string SqrOffvalue { get; set; }   // Square Off Value
            public string SLAbsOrticks { get; set; }   //    Stoploss Absolute or Ticks
            public string SLvalue { get; set; }   // Stoploss Value
            public string trailingSL { get; set; }   //  Trailing Stoploss
            public string tSLticks { get; set; }   //    Trailing Stoploss Ticks

            public BOOrderData(string order_refno, string strg_name, string user_apikey, string api_secret, string s_prdt_ali, string TokenNo, string exch, string Ttranstype, string Ret, string qty, string discqty, string Price, string ltpOratp,string SqrOffAbsOrtick, string SqrOffvalue, string SLAbsOrticks, string SLvalue,string trailingSL, string tSLticks)
            {
                this.order_refno = order_refno;
                this.strg_name = strg_name;
                this.user_apikey = user_apikey;
                this.api_secret = api_secret;
                this.s_prdt_ali = s_prdt_ali;
                this.TokenNo = TokenNo;
                this.exch = exch;
                this.Ttranstype = Ttranstype;
                this.Ret = Ret;
                this.qty = qty;
                this.discqty = discqty;
                this.Price = Price;
                this.ltpOratp = ltpOratp;
                this.SqrOffAbsOrtick = SqrOffAbsOrtick;
                this.SqrOffvalue = SqrOffvalue;
                this.SLAbsOrticks = SLAbsOrticks;
                this.SLvalue = SLvalue;
                this.trailingSL = trailingSL;
                this.tSLticks = tSLticks;
            }
        }

        public class PlaceMultiBOOrderRequest : Request
        {
            public BOOrderData[] orders { get; set; }

            public PlaceMultiBOOrderRequest(BOOrderData[] orders)
            {
                this.orders = orders;
            }
        }

        public class PlaceFOOptionsOrderRequest : Request
        {
            public string strg_name { get; set; }   //   Strategy Name
            public string spot_sym { get; set; }   //   Spot Symbol
            public string expiry_dt { get; set; }   //   Expiry Date
            public string opt_type { get; set; }   //   CE/PE
            public string Ttranstype { get; set; }   //   Transtaction Type
            public string prctyp { get; set; }   //   Price Type
            public string qty { get; set; }   //   Quantity
            public string Price { get; set; }   //     Price
            public string TrigPrice { get; set; }   //   Trigger Price
            public string PCode { get; set; }   //   Product Code
            public string strike_int { get; set; }   //   Strike
            public string offset { get; set; }   //    Off set

            public PlaceFOOptionsOrderRequest(string strg_name, string spot_sym, string expiry_dt, string opt_type, string Ttranstype, string prctyp, string qty, string Price, string TrigPrice, string PCode, string strike_int, string offset)
            {
                this.strg_name = strg_name;
                this.spot_sym = spot_sym;
                this.expiry_dt = expiry_dt;
                this.opt_type = opt_type;
                this.Ttranstype = Ttranstype;
                this.prctyp = prctyp;
                this.qty = qty;
                this.Price = Price;
                this.TrigPrice = TrigPrice;
                this.PCode = PCode;
                this.strike_int = strike_int;
                this.offset = offset;
            }
        }

        public class ExitBOOrderRequest : Request
        {
            public string uid { get; set; }   //  Client ID                        
            public string nestordernumber { get; set; } //  Nest Order Number
            public string SyomOrderId { get; set; } //  	Syom Order Id ( Leg order Indicator )
            public string status { get; set; } // Status of the order from Order book

            public ExitBOOrderRequest(string uid, string nestordernumber, string SyomOrderId, string status)
            {
                this.uid = uid;
                this.nestordernumber = nestordernumber;
                this.SyomOrderId = SyomOrderId;
                this.status = status;
            }
        }

        public class ExitCOOrderRequest : Request
        {
            public string uid { get; set; }   //  Client ID                        
            public string NestOrd { get; set; } //  Nest Order Number
            public string s_prdt_ali { get; set; } // Product Alias

            public ExitCOOrderRequest(string uid, string NestOrd, string s_prdt_ali)
            {
                this.uid = uid;
                this.NestOrd = NestOrd;
                this.s_prdt_ali = s_prdt_ali;
            }
        }

        public class ModifyOrderRequest : Request
        {
            public string uid { get; set; }   //  Client ID                        
            public string Actid { get; set; } //  Client ID Yes
            public string Exchangese { get; set; } // g Exchange Segment
            public string Tsym { get; set; } //  Trading Symbol
            public string Nstordno { get; set; } //  Nest Order Number
            public string Transtype { get; set; } //    Transaction type
            public string Prctype { get; set; } //  Price type
            public string Price { get; set; } //    Price
            public string Qty { get; set; } //  Quantity
            public string Dscqty { get; set; } //   Disclosed Quantity(Send as 0 for default)
            public string Trgprc { get; set; } //  Trigger price(Send as 0 for default)
            public string Validity { get; set; } //  Validity
            public string Symbol { get; set; } //   Scrip token
            public string Filledqty { get; set; } //  Filled shares
            public string Pcode { get; set; } //    Product code
            public string Mktpro { get; set; } //   Market protection flag (Should be sent as NA by default)
            public string DateDays { get; set; } //  Date Days (Should be sent as NA by default)
            public string s_prdt_ali { get; set; } // Product Alias

            public ModifyOrderRequest(string uid, string Actid, string Exchangese, string Tsym, string Nstordno, string Transtype, string Prctype, string Price, string Qty, string Dscqty, string Trgprc, string Validity, string Symbol, string Filledqty, string Pcode, string Mktpro, string DateDays, string s_prdt_ali)
            {
                this.uid = uid;
                this.Actid = Actid;
                this.Exchangese = Exchangese;
                this.Tsym = Tsym;
                this.Nstordno = Nstordno;
                this.Transtype = Transtype;
                this.Prctype = Prctype;
                this.Price = Price;
                this.Qty = Qty;
                this.Dscqty = Dscqty;
                this.Trgprc = Trgprc;
                this.Validity = Validity;
                this.Symbol = Symbol;
                this.Filledqty = Filledqty;
                this.Pcode = Pcode;
                this.Mktpro = Mktpro;
                this.DateDays = DateDays;
                this.s_prdt_ali = s_prdt_ali;
            }
        }        

        public class CancelOrderRequest : Request
        {
            public string uid { get; set; }   //  Client ID                        
            public string NestOrd { get; set; }   //  Nest Order Number
            public string sTradeSymbol { get; set; }   //  Trading Symbol
            public string sExch { get; set; }   //  Exchange Name 
            public string s_prdt_ali { get; set; } // Product Alias
            
            public CancelOrderRequest(string uid, string NestOrd, string sTradeSymbol, string sExch, string s_prdt_ali)
            {
                this.uid = uid;
                this.NestOrd = NestOrd;
                this.sTradeSymbol = sTradeSymbol;
                this.sExch = sExch;
                this.s_prdt_ali = s_prdt_ali;
            }
        }      

        public class OrderBookRequest : Request
        {
            public string s_prdt_ali { get; set; } // Product Alias
            public string uid { get; set; }   //  Client ID                        

            public OrderBookRequest(string s_prdt_ali, string uid)
            {
                this.uid = uid;
                this.s_prdt_ali = s_prdt_ali;
            }
        }

        public class TradeBookRequest : Request
        {
            public string s_prdt_ali { get; set; } // Product Alias
            public string uid { get; set; }   //  Client ID                        

            public TradeBookRequest(string s_prdt_ali, string uid)
            {
                this.uid = uid;               
                this.s_prdt_ali = s_prdt_ali;
            }
        }

        public class PositionBookRequest : Request
        {
            public string uid { get; set; }   //  Client ID
            public string actid { get; set; } // Client ID
            public string type { get; set; }  // DAY/NET
                                             // DAY is for day wise position
                                             // NET is for net wise position
            public string s_prdt_ali { get; set; } // Product Alias

            public PositionBookRequest(string uid, string actid, string type, string s_prdt_ali)
            {
                this.uid = uid;
                this.actid = actid;
                this.type = type;
                this.s_prdt_ali = s_prdt_ali;
            }
        }
        
        public class SquareOffPositionRequest : Request
        {
            public string uid { get; set; }  // Client ID;
            public string actid { get; set; }  // Client ID;
            public string s_prdt_ali { get; set; }  // Product Alias;
            public string Exchangeseg { get; set; }  // Exchange Segment;
            public string Pcode { get; set; }  // Product Code;
            public string Netqty { get; set; }  // Net Quantity;
            public string Token { get; set; }  // Token Number;
            public string Symbol { get; set; }  // Symbol Name;
            public string orderSource { get; set; }  // Order Source;

            public SquareOffPositionRequest(string uid, string actid, string s_prdt_ali, string Exchangeseg, string Pcode, string Netqty, string Token, string Symbol, string orderSource)
            {
                this.uid = uid;
                this.actid = actid;
                this.s_prdt_ali = s_prdt_ali;
                this.Exchangeseg = Exchangeseg;
                this.Pcode = Pcode;
                this.Netqty = Netqty;
                this.Token = Token;
                this.Symbol = Symbol;
                this.orderSource = orderSource;
            }
        }

        public class PartialPositionconvertionRequest : Request
        {
            public string uid { get; set; }  // Client ID;
            public string actid { get; set; }  // Client ID;
            public string s_prdt_ali { get; set; }  // Product Alias;
            public string Exchangeseg { get; set; }  // Exchange Segment;
            public string Pcode { get; set; }  // Product Code;
            public string Netqty { get; set; }  // Net Quantity;
            public string Token { get; set; }  // Token Number;
            public string Symbol { get; set; }  // Symbol Name;
            public string orderSource { get; set; }  // Order Source;

            public PartialPositionconvertionRequest(string uid, string actid, string s_prdt_ali, string Exchangeseg, string Pcode, string Netqty, string Token, string Symbol, string orderSource)
            {
                this.uid = uid;
                this.actid = actid;
                this.s_prdt_ali = s_prdt_ali;
                this.Exchangeseg = Exchangeseg;
                this.Pcode = Pcode;
                this.Netqty = Netqty;
                this.Token = Token;
                this.Symbol = Symbol;
                this.orderSource = orderSource;
            }
        }

        public class LTPCoverpercentageRequest : Request
        {
            public string Exchange { get; set; } // Exchange Name
            public string Symbol { get; set; } // Scrip Token
            public string Transtype { get; set; } // Tradnsaction Type

            public LTPCoverpercentageRequest(string Exchange, string Symbol, string Transtype)
            {
                this.Exchange = Exchange;
                this.Symbol = Symbol;
                this.Transtype = Transtype;
            }
        }

        public class FetchsymbolRequest : Request
        {
            public string s { get; set; } // 	Symbol Name from Watchlist
            public FetchsymbolRequest(string s)
            {
                this.s = s;
            }
        }

        public class SecurityInfoRequest : Request
        {
            public string uid { get; set; }      // Client ID
            public string Exchange { get; set; } // Exchange
            public string SrchFor { get; set; }  // SrchFor

            public SecurityInfoRequest(string uid, string Exchange, string SrchFor)
            {
                this.uid = uid;
                this.Exchange = Exchange;
                this.SrchFor = SrchFor;
            }
        }

        public class LimitsRequest : Request
        {
            public string uid { get; set; }          // Client ID
            public string actid { get; set; }        // Client ID
            public string segment { get; set; }      // Segment Type
            public string Exchange { get; set; }     // Exchange Name
            public string product { get; set; }      // Product Type


            public LimitsRequest(string uid, string actid, string segment, string Exchange, string product)
            {
                this.uid = uid;
                this.actid = actid;
                this.segment = segment;
                this.Exchange = Exchange;
                this.product = product;
            }
        }

        public class GetQuoteRequest : Request
        {
            public string Exchange { get; set; } // Exchange Exchange
            public string Symbol { get; set; } // Symbol  Symbol Name from Watchlist            

            public GetQuoteRequest(string Exchange, string Symbol)
            {
                this.Exchange = Exchange;
                this.Symbol = Symbol;
            }
        }

        public class FetchtokenRequest : Request
        {
            public string s { get; set; } // 	Symbol Name from Watchlist
            public FetchtokenRequest(string s)
            {
                this.s = s;
            }
        }

        public class LimitResponse
        {
            public string OpeningBalance;
            public string Netcashavailable;
            public string grossexposurevalue;
            public string segment;
            public string cncrealizedmtomprsnt;
            public string valueindelivery;
            public string cncsellcreditpresent;
            public string cncunrealizedmtomprsnt;
            public string deliverymarginprsnt;
            public string grossCollateral;
            public string BuyPower;
            public string PayoutAmt;
            public string cncbrokerageprsnt;
            public string turnover;
            public string unrealisedmtom;
            public string adhocscripmargin;
            public string specialmarginprsnt;
            public string BOmarginRequired;
            public string losslimit;
            public string IPOAmount;
            public string elm;
            public string cdsspreadbenefit;
            public string realisedmtom;
            public string category;
            public string nfospreadbenefit;
            public string Utilizedamount;
            public string exposuremargin;
            public string Adhoc;
            public string t1grossCollateral;
            public string directcollateralvalue;
            public string StockValuation;
            public string credits;
            public string branchadhoc;
            public string adhocmargin;
            public string additionalmarginprsnt;
            public string spanmargin;
            public string premiumpresent;
            public string BookedPNL;
            public string varmargin;
            public string marginScripBasketCustomPresent;
            public string tendermarginprsnt;
            public string cncmarginused;
            public string brokerageprsnt;
            public string UnbookedPNL;
            public string debits;
            public string COMarginRequired;
            public string multiplier;
            public string stat;
            public string mfamount;
            public string scripbasketmargin;
            public string buyExposurePrsnt;
            public string notionalcash;
            public string additionalpreexpirymarginprsnt;
            public string cncMarginElmPrsnt;
            public string PayinAmt;
            public string cncMarginVarPrsnt;
            public string mfssamountused;
            public string sellExposurePrsnt;
            public string Emsg;
        }


        public class FetchtokenResponse
        {
            public string stat;
            public string token;
            public string symbol;
            public string trading_symbol;
            public string Emsg;
        }

        public class FetchsymbolResponse
        {
            public string symbol_segment;
            public string symbol_product;
            public string symbol;
            public string symbol_token;
            public string trading_symbol;
            public string trading_symbol_desc;
            public string expiry_date;
            public string expiry_date_desc;
            public string strike_price;
            public string lot_size;
            public string seg_seq;
            public string order_seq;
            public string Emsg;
        }

        public class GetQuoteResponse
        {
            public int DecimalPrecision;
            public string Low;
            public int SQty;
            public string openPrice;
            public string TSymbl;
            public string TotalSell;
            public string yearlyLowPrice;
            public string Exp;
            public string PrvClose;
            public string vwapAveragePrice;
            public string LTD;
            public string yearlyHighPrice;
            public string exchFeedTime;
            public string TickSize;
            public int Multiplier;
            public string optiontype;
            public string TradeVolume;
            public string strikeprice;
            public string BRate;
            public string Change;
            public string LTP;
            public string LTQ;
            public string PerChange;
            public string TotalBuy;
            public string LTT;
            public string companyname;
            public string stat;
            public string Ltp;
            public string SRate;
            public int BQty;
            public int BodLotQty;
            public string Series;
            public string High;
            public string Emsg;
        }

        public class SquareOffPositionResponse
        {
            public string Result;            
            public string Emsg;
            public string stat;
        }

        public class PartialPositionconvertionResponse
        {
            public string Result;
            public string Emsg;
            public string stat;
        }

        public class LTPCoverpercentageResponse
        {
            public string TriggerPriceRange;
            public string Ltp;
            public string Decprc;
            public string Coverprc;
            public string stat;
            public string Emsg;
        }

        public class PositionBookResponse
        {
            public string unrealisedprofitloss;
            public string Fillsellqty;
            public string PriceNumerator;
            public string realisedprofitloss;
            public string Type;
            public string Fillbuyqty;
            public int BLQty;
            public string s_NetQtyPosConv;
            public string Sellavgprc;
            public string Exchangeseg;
            public string Opttype;
            public string Bqty;
            public string Exchange;
            public string Fillsellamt;
            public string actid;
            public string GeneralDenomenator;
            public string discQty;
            public string Instname;
            public string Netqty;
            public string sSqrflg;
            public string LTP;
            public string Tsym;
            public string Expdate;
            public string Buyavgprc;
            public string Netamt;
            public string Token;
            public string GeneralNumerator;
            public string companyname;
            public string stat;
            public string Sqty;
            public string PriceDenomenator;
            public string MtoM;
            public string Symbol;
            public string posflag;
            public string Series;
            public string BEP;
            public string Stikeprc;
            public string Pcode;
            public string Fillbuyamt;
            public string Emsg;
        }

        public class TradeBookResponse
        {
            public string series;
            public string AvgPrice;
            public string PriceNumerator;
            public string accountId;
            public string Custofrm;
            public string Ordduration;
            public string AlgoCategory;
            public string Filltime;
            public string BrokerClient;
            public string expdate;
            public string remarks;
            public string symbolname;
            public int Qty;
            public string ReportType;
            public string Exchange;
            public string Time;
            public string Prctype;
            public string usecs;
            public string Filldate;
            public string NOReqID;
            public string Exchtime;
            public string optiontype;
            public string GeneralDenomenator;
            public string ordergenerationtype;
            public string strikeprice;
            public int FillLeg;
            public long iSinceBOE;
            public string Nstordno;
            public string Tsym;
            public string GeneralNumerator;
            public string Expiry;
            public string companyname;
            public int bqty;
            public string FillId;
            public string stat;
            public string PriceDenomenator;
            public int Filledqty;
            public string panNo;
            public string Symbol;
            public string Trantype;
            public string posflag;
            public string Price;
            public string Exchseg;
            public string ExchordID;
            public string Pcode;
            public int Minqty;
            public string user;
            public string AlgoID;
            public int Fillqty;
            public string Emsg;
        }

        public class OrderBookResponse
        {
            public int Cancelqty;
            public string AlgoCategory;
            public string modifiedBy;
            public int RefLmtPrice;
            public string Mktpro;
            public int Qty;
            public string ExpSsbDate;
            public string Trgprc;
            public string ExchOrdID;
            public int COPercentage;
            public string RequestID;
            public string usecs;
            public string orderentrytime;
            public string Avgprc;
            public int Fillshares;
            public string optionType;
            public string ordergenerationtype;
            public int iSinceBOE;
            public string InstName;
            public string Nstordno;
            public int Unfilledsize;
            public string Usercomments;
            public string mpro;
            public string ticksize;
            public string strikePrice;
            public string OrderedTime;
            public string panNo;
            public string defmktproval;
            public string ExchConfrmtime;
            public int Minqty;
            public string AlgoID;
            public string ordersource;
            public string sipindicator;
            public string decprec;
            public string series;
            public string PriceNumerator;
            public string Ordvaldate;
            public string Trsym;
            public string accountId;
            public int Dscqty;
            public string BrokerClient;
            public string reporttype;
            public string remarks;
            public string Scripname;
            public string Prc;
            public string Exchange;
            public string Sym;
            public string token;
            public string Prctype;
            public string RejReason;
            public string GeneralDenomenator;
            public string Validity;
            public string GeneralNumerator;
            public string stat;
            public string multiplier;
            public string bqty;
            public string noMktPro;
            public string discQtyPerc;
            public string Status;
            public string PriceDenomenator;
            public string SyomOrderId;
            public string Trantype;
            public string ExpDate;
            public string marketprotectionpercentage;
            public string Pcode;
            public string Exseg;
            public string user;
            public string Emsg;
        }
        

        public class CancelOrderResponse
        {
            public string Result;
            public string stat;
            public string Emsg;
        }

        public class ModifyOrderResponse
        {
            public string NOrdNo;
            public string stat;
            public string Emsg;
        }

        public class ExitCOOrderResponse
        {            
            public string stat;
            public string Emsg;
        }

        public class ExitBOOrderResponse
        {
            public string stat;
            public string Emsg;
        }

        public class PlaceFOOptionsOrderResponse
        {
            public string stat;
            public string NOrdNo;
            public string Emsg;
        }

        public class PlaceMultiBOOrderResponse
        {
            public string stat;
            public string nestOrderNumber;
            public string Emsg;
        }

        public class PlaceBOOrderResponse
        {
            public string stat;
            public string nestOrderNumber;
            public string Emsg;
        }

        public class PlaceMultiOrderResponse
        {
            public string stat;
            public string NOrdNo;
            public string Emsg;
        }

        public class PlaceOrderResponse
        {
            public string stat;
            public string NOrdNo;
            public string Emsg;
        }

        public class SecurityInfoResponse
        {
            public string IntPayDate;
            public string PriceQuatation;
            public double SellCryCost;
            public int Blq;
            public string HCircuitLimit;
            public double BuyCryCost;
            public string IMSpreadBenefit;
            public double Totalopeninterest;
            public string DeliveryUnits;
            public string DeliveryEndDate;
            public string OtherSellMargin;
            public string NoDelEndTime;
            public string NoDelStartDate;
            public string TickSize;
            public string ExpiryDte;
            public string OtherBuyMargin;
            public string Comments;
            public string TenderPeriod;
            public string ListingDate;
            public string Symbol;
            public string Series;
            public string CreditRating;
            public string Exposuremargin;
            public string LocalUpdateTime;
            public string IssedCapital;
            public string PriceNumerator;
            public string IsuStartDate;
            public int PermitedtoTrade;
            public string TenderPeriodEnd;
            public int Warning;
            public string Remarks;
            public string DPR1;
            public string BuyVarMargin;
            public string DPR2;
            public string BookClsStartTime;
            public int Perchgopeninterest;
            public string ExpulsionDate;
            public string Exchange;
            public int Issuerate;
            public string QuantityUnit;
            public string ReAdminDate;
            public string SellVarMargin;
            public string TenderPeriodStart;
            public string GeneralDenomenator;
            public double openinterest;
            public string ExchSeg;
            public string Spotprc;
            public string MarketType;
            public string RecordDate;
            public string ValueTradedToday;
            public string LTT;
            public string PriceUnits;
            public string GeneralNumerator;
            public string MaxOrderSize;
            public string stat;
            public string PriceDenomenator;
            public string DeliveryStartDate;
            public string LastTradingDate;
            public string LCircuitLimit;
            public int Freeze;
            public string Instrument;
            public string IsuMaturityDate;
            public string Emsg;
        }

        public class ErrorDetailsResponse
        {
            public string error_no;
            public string error_msg;
            public string error_type;
        }

        public class ErrorResponse
        {            
            public string status;            
            public ErrorDetailsResponse response;
        }
        

    //private string key;
    //private string secret;
    //
    //
    //                         // https://abapi.algomojo.com/1.0/
    //private const String URL = "https://tjapi.algomojo.com/1.0/";
    //                         // https://zbapi.algomojo.com/1.0/

    private Config config;

        public API(Config config)
        {
            this.config = config;
        }

        private static string GetContent(HttpWebResponse response)
        {
            string content = "";
            using (Stream responseStream = response.GetResponseStream())
            {
                using (StreamReader reader = new StreamReader(responseStream))
                {
                    content = reader.ReadToEnd();
                    reader.Close();
                }
                responseStream.Close();
            }           
            return content;
        }

        private string SendRequest(InternalRequest internalRequest, String verb)
        {
            JsonSerializer serializer = new JsonSerializer();

            StringBuilder sb = new StringBuilder();
            using (StringWriter sw = new StringWriter(sb))
            {
                using (JsonWriter writer = new JsonTextWriter(sw))
                {
                    serializer.Serialize(writer, internalRequest);
                }
            }

            HttpWebRequest request2 = (HttpWebRequest)WebRequest.Create(config.GetURL() + verb);
            request2.AutomaticDecompression = DecompressionMethods.GZip | DecompressionMethods.Deflate;
            request2.ServicePoint.Expect100Continue = false;
            request2.AllowAutoRedirect = false;
            request2.UserAgent = UserAgent;
            request2.Headers.Add("Upgrade-Insecure-Requests", "1");
            request2.Headers.Add("Accept-Language", "en-US,en;q=0.9");
            request2.Accept = "application/json";
            request2.ContentType = "application/json";
            request2.Method = "POST";

            using (Stream request2Stream = request2.GetRequestStream())
            {
                byte[] body = Encoding.ASCII.GetBytes(sb.ToString());
                request2Stream.Write(body, 0, body.Length);
                request2Stream.Close();
            }

            using (HttpWebResponse response2 = (HttpWebResponse)request2.GetResponse())
            {
                return GetContent(response2);
            }
        }

        public void PlaceOrder(PlaceOrderRequest request, out ErrorResponse error_info, out PlaceOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PlaceOrder");

            result = JsonConvert.DeserializeObject<PlaceOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void PlaceMultiBOOrder(PlaceMultiOrderRequest request, out ErrorResponse error_info, out PlaceMultiOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PlaceMultiOrder");

            result = JsonConvert.DeserializeObject<PlaceMultiOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void PlaceBOOrder(PlaceBOOrderRequest request, out ErrorResponse error_info, out PlaceBOOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PlaceBOOrder");

            result = JsonConvert.DeserializeObject<PlaceBOOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void PlaceMultiBOOrder(PlaceMultiBOOrderRequest request, out ErrorResponse error_info, out PlaceMultiBOOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PlaceMultiBOOrder");

            result = JsonConvert.DeserializeObject<PlaceMultiBOOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void PlaceFOOptionsOrder(PlaceFOOptionsOrderRequest request, out ErrorResponse error_info, out PlaceFOOptionsOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PlaceFOOptionsOrder");

            result = JsonConvert.DeserializeObject<PlaceFOOptionsOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }
        
        public void ExitBOOrder(ExitBOOrderRequest request, out ErrorResponse error_info, out ExitBOOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "ExitBOOrder");

            result = JsonConvert.DeserializeObject<ExitBOOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void ExitCOOrder(ExitCOOrderRequest request, out ErrorResponse error_info, out ExitCOOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "ExitCOOrder");

            result = JsonConvert.DeserializeObject<ExitCOOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void ModifyOrder(ModifyOrderRequest request, out ErrorResponse error_info, out ModifyOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "ModifyOrder");

            result = JsonConvert.DeserializeObject<ModifyOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void CancelOrder(CancelOrderRequest request, out ErrorResponse error_info, out CancelOrderResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "CancelOrder");

            result = JsonConvert.DeserializeObject<CancelOrderResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void OrderBook(OrderBookRequest request, out ErrorResponse error_info, out OrderBookResponse[] result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "OrderBook");

            result = JsonConvert.DeserializeObject<OrderBookResponse[]>(content);

            if (content.StartsWith("{"))
                error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
            else
                error_info = new ErrorResponse();
        }

        public void TradeBook(TradeBookRequest request, out ErrorResponse error_info, out TradeBookResponse[] result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "TradeBook");

            result = JsonConvert.DeserializeObject<TradeBookResponse[]>(content);

            if (content.StartsWith("{"))
                error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
            else
                error_info = new ErrorResponse();
        }

        public void PositionBook(PositionBookRequest request, out ErrorResponse error_info, out PositionBookResponse[] result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PositionBook");

            result = JsonConvert.DeserializeObject<PositionBookResponse[]>(content);

            if (content.StartsWith("{"))
                error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
            else
                error_info = new ErrorResponse();
        }

        public void SquareOffPosition(SquareOffPositionRequest request, out ErrorResponse error_info, out SquareOffPositionResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "SquareOffPosition");

            result = JsonConvert.DeserializeObject<SquareOffPositionResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void PartialPositionconvertion(PartialPositionconvertionRequest request, out ErrorResponse error_info, out PartialPositionconvertionResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "PartialPositionconvertion");

            result = JsonConvert.DeserializeObject<PartialPositionconvertionResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void LTPCoverpercentage(LTPCoverpercentageRequest request, out ErrorResponse error_info, out LTPCoverpercentageResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "LTPCoverpercentage");

            result = JsonConvert.DeserializeObject<LTPCoverpercentageResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void Fetchsymbol(string symbol, out ErrorResponse error_info, out FetchsymbolResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = new FetchsymbolRequest(symbol);

            string content = SendRequest(internalRequest, "fetchsymbol");

            result = new FetchsymbolResponse();
            FetchsymbolResponse[] array_result = JsonConvert.DeserializeObject<FetchsymbolResponse[]>(content);
            if (array_result != null)
            {
                if (array_result.Length > 0)
                    result = array_result[0];
            }

            if (content.StartsWith("{"))
                error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
            else
                error_info = new ErrorResponse();
        }

        public void SecurityInfo(SecurityInfoRequest request, out ErrorResponse error_info, out SecurityInfoResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "SecurityInfo");

            result = JsonConvert.DeserializeObject<SecurityInfoResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void Limits(LimitsRequest request, out ErrorResponse error_info, out LimitResponse result)
        {            
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "Limits");
                
            result = JsonConvert.DeserializeObject<LimitResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);            
        }

        public void GetQuote(GetQuoteRequest request, out ErrorResponse error_info, out GetQuoteResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = request;

            string content = SendRequest(internalRequest, "GetQuote");

            result = JsonConvert.DeserializeObject<GetQuoteResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }

        public void Fetchtoken(string symbol, out ErrorResponse error_info, out FetchtokenResponse result)
        {
            InternalRequest internalRequest = new InternalRequest();
            internalRequest.api_key = config.Key;
            internalRequest.api_secret = config.Secret;
            internalRequest.data = new FetchtokenRequest(symbol);

            string content = SendRequest(internalRequest, "fetchtoken");

            result = JsonConvert.DeserializeObject<FetchtokenResponse>(content);
            error_info = JsonConvert.DeserializeObject<ErrorResponse>(content);
        }
    }
}

